package de.unimarburg.swt.emf.fix.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.refactor.common.core.EmfRefactoring;

import de.unimarburg.swt.emf.fix.extensionpoints.ExtensionPoints;
import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;

/**
 * Utility class for loading the relation definitions from EMF Fix'es Extension Points
 * 
 * @author Matthias Burhenne
 *
 */
public class RelationLoader {

	/**
	 * Loads the relations between model refactorings and their caused smells.
	 * 
	 * @return HashMap<EmfRefactoring, Set<ModelSmell>> - map representing the relation
	 */
	public static HashMap<EmfRefactoring, Set<ModelSmell>> loadRefactoringToSmellsRelation() {
		HashMap<EmfRefactoring, Set<ModelSmell>> relationMap = new HashMap<EmfRefactoring, Set<ModelSmell>>();
		IConfigurationElement[] configElements = Platform.getExtensionRegistry().getConfigurationElementsFor(ExtensionPoints.EXTENSION_POINT_NAME_REFACTORING_TO_SMELLS);
		for (IConfigurationElement element : configElements) {
			if (element.getName().equals(ExtensionPoints.REFACTORING_TO_SMELL_TAG)) {					
				final String refactoringID = element.getAttribute(ExtensionPoints.REFACTORING_ID_TAG);
				HashSet<String> smellIDs = new HashSet<String>();
				for(IConfigurationElement relationElementChild : element.getChildren()){
					if(relationElementChild.getName().equals(ExtensionPoints.REFACTORING_TO_SMELLS_SMELL_ELEMENT_TAG)){
						smellIDs.add(relationElementChild.getAttribute(ExtensionPoints.SMELL_ID_TAG));
					}
				}
				EmfRefactoring refactoring = getRefactoringById(refactoringID);
				for(String smellIDFromList : smellIDs){
					ModelSmell smell = ModelSmell.getSmell(smellIDFromList);
					if(refactoring != null && smell != null){
						if(relationMap.containsKey(refactoring)){
							relationMap.get(refactoring).add(smell);
						}else{
							HashSet<ModelSmell> value = new HashSet<ModelSmell>();
							value.add(smell);
							relationMap.put(refactoring, value);
						}
					}
				}
			}
		}
		return relationMap;
	}

	/**
	 * Loads the relations between model smells and their fixing smells.
	 * 
	 * @return HashMap<ModelSmell, Set<EmfRefactoring>> - map representing the relation
	 */
	public static HashMap<ModelSmell, Set<EmfRefactoring>> loadSmellToRefactoringsRelation() {
		HashMap<ModelSmell, Set<EmfRefactoring>> relationMap = new HashMap<ModelSmell, Set<EmfRefactoring>>();
		IConfigurationElement[] configElements = Platform.getExtensionRegistry().getConfigurationElementsFor(ExtensionPoints.EXTENSION_POINT_NAME_SMELL_TO_REFACTORINGS);
		for(IConfigurationElement element : configElements){
			if(element.getName().equals(ExtensionPoints.SMELL_TO_REFACTORING_TAG)){
				final String smellID = element.getAttribute(ExtensionPoints.SMELL_ID_TAG);
				HashSet<String> refactoringIDs = new HashSet<String>();
				for(IConfigurationElement relationElementChild : element.getChildren()){
					if(relationElementChild.getName().equals(ExtensionPoints.SMELL_TO_REFACTORINGS_REFACTORING_ELEMENT_TAG)){
						refactoringIDs.add(relationElementChild.getAttribute(ExtensionPoints.REFACTORING_ID_TAG));
					}
				}
				ModelSmell smell = ModelSmell.getSmell(smellID);
				for(String refactoringID : refactoringIDs){
					EmfRefactoring refactoring = getRefactoringById(refactoringID);
				
					if(refactoring != null && smell != null){
						if(relationMap.containsKey(smell)){
							relationMap.get(smell).add(refactoring);
						}else{
							HashSet<EmfRefactoring> value = new HashSet<EmfRefactoring>();
							value.add(refactoring);
							relationMap.put(smell, value);
						}
					}
				}
			}
		}
		return relationMap;
	}
	
	/*
	 * Helper method getting a refactoring by its id. Returns null if the id is not in use.
	 */
	private static EmfRefactoring getRefactoringById(String id){
		EmfRefactoring refactoring;
		try{
			refactoring = EmfRefactoring.getById(id);
		}catch(IllegalArgumentException ex){
//			ex.printStackTrace();
			return null;
		}
		return refactoring;
	}

}
