package de.unimarburg.swt.emf.metrics;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.unimarburg.swt.emf.metrics.calculator.MetricCalculator;
import de.unimarburg.swt.emf.metrics.generator.MetricGenerator;
import de.unimarburg.swt.emf.metrics.gui.view.ResultsView;
import de.unimarburg.swt.emf.metrics.model.Metric;
import de.unimarburg.swt.emf.metrics.model.calculator.Configuration;
import de.unimarburg.swt.emf.metrics.model.calculator.Result;
import de.unimarburg.swt.emf.metrics.model.generator.MetricInfo;
import de.unimarburg.swt.emf.metrics.operations.IOperation;
import de.unimarburg.swt.emf.metrics.operations.Operations;
import de.unimarburg.swt.emf.metrics.utils.DependenciesFile;
import de.unimarburg.swt.emf.metrics.utils.MetricLoader;
import de.unimarburg.swt.emf.metrics.utils.xml.XMLPluginFile;
import de.unimarburg.swt.emf.metrics.utils.xml.XMLProjectFile;

/**
 * This class is the main controller. It provides Methods needed to coordinate all 
 * EMFMetrics system tasks. 
 * 
 * @author Pawel Stepien
 */
public class EMFMetrics {

	private static LinkedList<Configuration> configurations;
	private static MetricCalculator calculator;
	private static MetricGenerator generator;
	private static LinkedList<Metric> allMetrics=null;
	private static LinkedList<Result> resultsViewInput;
	private static TableViewer resultsViewer=null;
	
	
	/**
	 * The constructor that creates a new <i>EMFMetrics</i> controller class.
	 */
	public EMFMetrics() {
		configurations = new LinkedList<Configuration>();
		calculator = MetricCalculator.getCalculator();
		generator = MetricGenerator.getGenerator();
		resultsViewInput = new LinkedList<Result>();
	}
	
	/**
	 * Setter method for the resultsViewer attribute.
	 * 
	 * @param tableViewer
	 */
	public static void setResultsViewer(TableViewer viewer){
		resultsViewer = viewer;
	}
	
	/**
	 * Returns the actual input from the <i>ResultsView</i>
	 * 
	 * @return ResultsView input
	 */
	public static LinkedList<Result> getResultsViewInput(){
		return resultsViewInput;
	}
	
	/**
	 * Returns the operation class with the given name, 
	 * which implements the <i>IOperation</i> interface.
	 * 
	 * @param operation name
	 * 
	 * @return operation class
	 */
	public static IOperation getOperation(String name){
		return Operations.getOperation(name);
	}
	
	/**
	 * Returns an <i>String</i> array containing the names of all supported operations.
	 * 
	 * @return names of all supported operations.
	 */
	public static String[] getOperationNames(){
		return Operations.getNames();
	}
	
	/**
	 * Returns an <i>String</i> array containing the names of all supported operations.
	 * 
	 * @return names of all supported operations.
	 */
	public static LinkedList<Metric> getAllMetrics() {
		if(allMetrics==null)
			allMetrics = MetricLoader.loadMetrics();
		return allMetrics;
	}
	
		
	
	/**
	 * Saves the actual Metric configuration for a given project into a File.
	 * 
	 * @param project
	 */
	public static void saveConfiguration(IProject project) {
		XMLProjectFile.saveConfigurator(getConfiguration(project));
	}

	/**
	 * Loads the actual Metric configuration for a given project from a File.
	 * 
	 * @param project
	 */
	public static Configuration loadConfiguration(IProject project) {
		return XMLProjectFile.loadConfigurator(project);
	}
	
	/**
	 * Returns a LinkedList containing all selected metrics for a given project.
	 * 
	 * @param project
	 * 
	 * @return selectedMetrics
	 */
	public static LinkedList<Metric> getSelectedMetrics(IProject project){
		Configuration configuration = getConfiguration(project);
		return configuration.getSelectedMetrics();
	}
	
	/**
	 * Saves a metric selection for a given project.
	 * 
	 * @param project
	 * @param metrics selection
	 */
	public static void setConfiguration(IProject project, boolean[] selection){
		Configuration configuration= getConfiguration(project);
		configuration.setSelection(selection);
	}
	
	/**
	 * Calculates all metrics configured for a given project. 
	 * After all calculations are done the results are being added to the <i>ResultsView</i>.
	 * 
	 * @param project
	 * @param context for the calculations
	 */
	public static void calculateConfiguredMetrics(IProject project, List<EObject> context){
		Configuration configuration=getConfiguration(project);
		calculator.setConfiguration(configuration);
		calculator.setContext(context);
		calculator.calculateMetrics();
		resultsViewInput.addAll(calculator.getResults());
		if(!isMetricsViewOpen()){
			createMetricsView();
		}
		if(resultsViewer!=null)
			if(resultsViewInput!=null)
				resultsViewer.setInput(resultsViewInput);
	}
		
	/**
	 * Creates a new metric using a <i>MetricInfo</i> class. This contains the following actions:
	 * <br>- generation of a new metric class which implements the <i>ICOmpositeCalculateClass</i> interface
	 * <br>- generation of a metric extension in the plugin.xml file of the target project
	 * <br>- updating of the class dependencies of the target project
	 * 
	 * @param progress monitor
	 * @param metric info
	 * @param target project
	 */
	public static void createNewMetric(IProgressMonitor monitor, MetricInfo info, IProject project){
		generator.setMetricInfo(info);
		generator.createCalculateClass(monitor);
		DependenciesFile.updateDependencies(info.getProjectName());
		XMLPluginFile.createMetricEntry(info.getProjectPath(), info.getName(), info.getId(), 
				info.getDescription(), info.getMetamodel(), info.getContext(), info.getValueType(), info.getPackage()+"."+info.getClassName());
		try {
			project.refreshLocal(IProject.DEPTH_INFINITE, monitor);
			project.refreshLocal(IProject.DEPTH_INFINITE, monitor);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns a LinkedList containing all selected metrics for a given project that are
	 * defined for the given metamodel and the given context.
	 * 
	 * @param metamodel
	 * @param context
	 * 
	 * @return filteredMetrics
	 */
	public static LinkedList<Metric> getFilteredMetrics(String metamodel, String context) {
		LinkedList<Metric> result = new LinkedList<Metric>();
		for (Metric metric : allMetrics) {
			if (metric.getMetamodel().equals(metamodel)
					&& metric.getContext().equals(context)
//					&& metric.getCalculateClass() instanceof ICompositeCalculateClass
					){
				result.add(metric);
				}
		}
		return result;
	}
		
	private static void createMetricsView(){
		IWorkbench wb = PlatformUI.getWorkbench();
		IWorkbenchWindow win = wb.getActiveWorkbenchWindow();
		IWorkbenchPage page = win.getActivePage();
			try{
				page.showView(ResultsView.ID);
			} catch (PartInitException e) {
				e.printStackTrace();
			}
	}
	
	private static boolean isMetricsViewOpen(){
		IWorkbench wb = PlatformUI.getWorkbench();
		IWorkbenchWindow win = wb.getActiveWorkbenchWindow();
		IWorkbenchPage page = win.getActivePage();
		for(int i=0; i<page.getViewReferences().length; i++){
			if(page.getViewReferences()[i].getId().equals(ResultsView.ID))
				return true;
		}
		return false;
	}

	private static Configuration getConfiguration(IProject project) {
		Configuration configuration = null;
		if(!configurations.isEmpty())
		for (Configuration tempConfiguration : configurations) {
			if (tempConfiguration.getProject().equals(project))
				configuration=tempConfiguration;
		}
		if(configuration == null)
			configuration = loadConfiguration(project);
		if(configuration == null){
			configuration = new Configuration(project);	
		}
		configurations.add(configuration);
		return configuration;
	}

	public static void calculateAllConfiguredMetrics(IProject project, EObject root) {
		Configuration configuration=getConfiguration(project);
		calculator.setConfiguration(configuration);
		LinkedList<Result> results = new LinkedList<Result>();
		LinkedList<EObject> allEObjects = new LinkedList<EObject>();
		allEObjects.add(root);
		allEObjects.addAll(getAllEObjects(root));
		startTimeRecording();
		for(EObject currentEObject : allEObjects){
			ArrayList<EObject> contextObjectList = new ArrayList<EObject>();
			contextObjectList.add(currentEObject);
			calculator.setContext(contextObjectList);
			calculator.calculateMetrics();
			results.addAll(calculator.getResults());
		}
		stopTimeRecording();
		System.out.println("Calculated results: " + results.size());
		if(!isMetricsViewOpen()){
			createMetricsView();
		}
		if(resultsViewer!=null)
			if(resultsViewInput!=null)
				resultsViewInput.addAll(results);
				resultsViewer.setInput(resultsViewInput);
	}

	private static List<EObject> getAllEObjects(EObject root) {
		ArrayList<EObject> eObjects = new ArrayList<EObject>();
//		List<EObject> directContents = root.eContents();
//		eObjects.addAll(directContents);
//		for(EObject contentObject : directContents){
//			eObjects.addAll(getAllEObjects(contentObject));
//		}
		TreeIterator<EObject> iter = root.eAllContents();
		while (iter.hasNext()) {
			eObjects.add(iter.next());
		}
		return eObjects;
	}
	
	static long timebegin;
	static long timeend;
	
	private static void startTimeRecording() {
		timebegin = System.currentTimeMillis();
	}

	private static void stopTimeRecording() {
		timeend = System.currentTimeMillis();
		System.out.println("Time needed: " + ((timeend - timebegin)) + " ms");
	}
}
