package de.unimarburg.swt.emf.metrics.generator;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.emf.codegen.util.CodeGenUtil;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.osgi.util.ManifestElement;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import org.osgi.framework.Constants;

import de.unimarburg.swt.emf.metrics.Activator;
import de.unimarburg.swt.emf.metrics.model.generator.CompositeMetricInfo;
import de.unimarburg.swt.emf.metrics.model.generator.HenshinMetricInfo;
import de.unimarburg.swt.emf.metrics.model.generator.MetricInfo;
import de.unimarburg.swt.emf.metrics.model.generator.OCLMetricInfo;

/**
 * This class provides a controller responsible for the metric generation process.
 * This is a singleton class. The <i>getGenerator()</i> method returns the instance of his class.  
 * 
 * @author Pawel Stepien
 */
public class MetricGenerator {

	private static final String OCL_CALCULATE_CLASS_TEMPLATE = "OCLCalculateClassTemplate";
	private static final String HENSHIN_TEMPLATE_CLASS_NAME = "HenshinCalculateClassTemplate";
	private static final String COMPOSITE_TEMPLATE_CLASS_NAME = "SimpleCompositeCalculateClassTemplate"; //"CompositeCalculateClassTemplate";
	private static final String SKELETON_TEMPLATE_CLASS_NAME = "SkeletonCalculateClassTemplate";
	private static final  String TEMPLATE_DIR = "/templates";
	private static final String SOURCE_DIR = "/src/";
	private static final  String TEMPLATE_FILE_EXTENSION = ".javajet";
	private static final String JAVA_FILE_EXTENSION = ".java";
	private static final String PLUGINSPATH = Platform.getInstallLocation().getURL().getPath() + "plugins/";
	private static final String BUNDLEVERSION = "Bundle-Version";
	private static MetricGenerator generator;
	private static String templateDirectory;
	private static List<IClasspathEntry> classpathEntries;
	private MetricInfo metricInfo;
	//TODO delete
	private static final String EMFMETRICSJAR = "EMFMETRICS_JAR";	

	/**
	 * Returns the instance of this class.
	 * 
	 * @return metricGenerator
	 */
	public static MetricGenerator getGenerator(){
		if(generator==null)
			generator = new MetricGenerator();
		return generator;
	}
	
	
	
	/**
	 * Sets the <i>MetricInfo</i> object that provides all information needed
	 * for the metric generation.
	 * 
	 * @param metricInfo
	 */
	public void setMetricInfo(MetricInfo info){
		this.metricInfo = info;
	}
		
	/**
	 * Generates a new metric <i>ICompositeCalculateClass</i> class.
	 * 
	 * @return progressMonitor
	 */
	public void createCalculateClass(IProgressMonitor monitor) {
		String generatedCode = "";
		String templateName = "";
		if(this.metricInfo instanceof HenshinMetricInfo){
			templateName = HENSHIN_TEMPLATE_CLASS_NAME;
		}
		if(this.metricInfo instanceof CompositeMetricInfo){
			templateName = COMPOSITE_TEMPLATE_CLASS_NAME;
		}
		if(this.metricInfo instanceof OCLMetricInfo){
			templateName = OCL_CALCULATE_CLASS_TEMPLATE;
		}
		if(this.metricInfo instanceof MetricInfo && !(this.metricInfo instanceof OCLMetricInfo) && !(this.metricInfo instanceof HenshinMetricInfo) && !(this.metricInfo instanceof CompositeMetricInfo)){
			templateName = SKELETON_TEMPLATE_CLASS_NAME;
		}
		try {
			generatedCode = generateCode(monitor, templateName);
		    saveCode(monitor, generatedCode, this.metricInfo.getClassName());
		} catch (JETException e) {
			e.printStackTrace();
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
		
	private MetricGenerator() {
		templateDirectory = setTemplateDirectory();
//		classpathEntries = getVariableEntries();
		//TODO
		classpathEntries = new ArrayList<IClasspathEntry>();
		setClassPathEntries();
	}
	
	//TODO use instead of getVariableEntries()
	private void setClassPathEntries() {
    Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID);
    String version = (String) bundle.getHeaders().get(BUNDLEVERSION);
    classpathEntries.add(JavaCore.newLibraryEntry(new Path(PLUGINSPATH+ Activator.PLUGIN_ID + "_" + version + ".jar"), null, null));
	}
	
	//TODO delete
	private ArrayList<IClasspathEntry> getVariableEntries() {
		ArrayList<IClasspathEntry> classpathEntries = new ArrayList<IClasspathEntry>();
		Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID); 
		String requires = (String) bundle.getHeaders().get(Constants.BUNDLE_CLASSPATH);
		ManifestElement[] elements = null;
		try {
			elements = ManifestElement.parseHeader(Constants.BUNDLE_CLASSPATH, requires);
		} catch (BundleException e) {
			e.printStackTrace();
		}
		ManifestElement element = elements[0];
		URL url = bundle.getEntry(element.getValue());
		if (url != null) {
			try {
				IPath path = new Path(FileLocator.toFileURL(url).getFile());
				if (!path.equals(JavaCore.getClasspathVariable(EMFMETRICSJAR))) {
					JavaCore.setClasspathVariable(EMFMETRICSJAR, path,	null);
				}
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (JavaModelException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			classpathEntries.add(JavaCore.newVariableEntry(new Path(EMFMETRICSJAR), null, null));
		}
		return classpathEntries;
	}
	
	private String generateCode(IProgressMonitor monitor, String template) {
		String templatePath = templateDirectory + template + TEMPLATE_FILE_EXTENSION;
		ClassLoader classLoader = getClass().getClassLoader();
		JETEmitter jetEmitter = new JETEmitter(templatePath, classLoader);
		jetEmitter.getClasspathEntries().addAll(classpathEntries);
		String result = "";
		IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 1);
		try {
			result = jetEmitter.generate(subMonitor, new Object[] {this.metricInfo});
		} catch (JETException e) {
			e.printStackTrace();
		}
		System.out.println(result);
		return result;
	}
	
	private void saveCode(IProgressMonitor monitor, String content, String fileName) throws CoreException, JETException {
		IContainer container = findOrCreatePackage(monitor);
		if (container == null) {
			throw new JETException("Could not find or create container for package " + this.metricInfo.getPackage() + " in " + this.metricInfo.getProjectName());
		}
		IFile file = container.getFile(new Path(fileName + JAVA_FILE_EXTENSION));
		System.out.println("saving code into: "+file.getLocation());
		InputStream inputStream = new ByteArrayInputStream(content.getBytes());
		if (!file.exists()) {
			file.create(inputStream, false, monitor);
		} else {
			container.refreshLocal(1, monitor);
			file.setContents(inputStream, true, false, monitor);
		}
	}
	
	private IContainer findOrCreatePackage(IProgressMonitor progressMonitor) throws CoreException {
		IPath outputPath = new Path(this.metricInfo.getProjectName() + SOURCE_DIR + this.metricInfo.getPackage().replace('.', '/'));
		IProgressMonitor subMonitor = new SubProgressMonitor(progressMonitor, 1);
		IPath localLocation = null; 
		IContainer container = CodeGenUtil.EclipseUtil.findOrCreateContainer(outputPath, true, localLocation, subMonitor);
		return container;
	}

	private String setTemplateDirectory() {
		String directory = "";
		final Bundle bundle = Activator.getDefault().getBundle();
		try {
			directory = FileLocator.toFileURL(bundle.getEntry(TEMPLATE_DIR)).getFile();
		} catch (final IOException e) {
			e.printStackTrace();
		}
		return directory;
	}
	
}
