package de.unimarburg.swt.emf.modelsmell.generator;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.emf.codegen.util.CodeGenUtil;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.osgi.util.ManifestElement;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import org.osgi.framework.Constants;

import de.unimarburg.swt.emf.modelsmell.Activator;
import de.unimarburg.swt.emf.modelsmell.model.generator.GenerationType;
import de.unimarburg.swt.emf.modelsmell.model.generator.ModelSmellInfo;

/**
 * Central controller class of the generation of finder classes. It consumes <i>ModelSmellInfo</i> objects and calls
 * <br>- JET with the appropriate templates.
 * 
 * @author Matthias Burhenne
 *
 */

public class ModelSmellGenerator {
	private static List<IClasspathEntry> classpathEntries;
	private static final String EMFMODELSMELLSJAR = "EMFMODELSMELLS_JAR";
	private static final String FRAME_TEMPLATE_CLASS_NAME = "FrameFinderClassTemplate";
	private static ModelSmellGenerator generator = null;
	private static final String HENSHIN_TEMPLATE_CLASS_NAME = "HenshinFinderClassTemplate";
	private static final String JAVA_FILE_EXTENSION = ".java";
	private static final String METRIC_TEMPLATE_CLASS_NAME = "MetricBasedFinderClassTemplate";
	private static final String SOURCE_DIR = "/src/";
	private static final  String TEMPLATE_DIR = "/templates";
	private static final  String TEMPLATE_FILE_EXTENSION = ".javajet";
	/**
	 * returns the current instance of <i>ModelSmellGenerator</i>.
	 * 
	 * @return current generator instance
	 */
	public static ModelSmellGenerator getGenerator(){
		if(generator == null){
			generator = new ModelSmellGenerator();
		}
		return generator;
	}
	private ModelSmellInfo info;
	
	//===================

	private String templateDirectory;
	protected final String BUNDLEVERSION = "Bundle-Version";
	
	protected final String PLUGINSPATH = 
		Platform.getInstallLocation().getURL().getPath() + "/plugins/"; //"plugins/"; 
	//====================
	
	private ModelSmellGenerator(){
		templateDirectory = setTemplateDirectory();
		classpathEntries = getVariableEntries();
	}

	/**
	 * Selects the appropriate JET template from the info attribute of this class.
	 * <br>- Generates the code for the finder class and writes it to the target project.
	 * <br>- Also creates the needed target package if not existant.
	 * @param monitor - progressmonitor
	 */
	public void createFinderClass(IProgressMonitor monitor) {
		String generatedCode = "";
		String templateName = "";
		if(this.info instanceof ModelSmellInfo){
			switch (info.getGenerationType()) {
			case SKELETON:
				templateName = FRAME_TEMPLATE_CLASS_NAME;
				break;
			case HENSHIN_TRANSFORMATION:
				templateName = HENSHIN_TEMPLATE_CLASS_NAME;
				break;
			case METRIC_BASED:
				templateName = METRIC_TEMPLATE_CLASS_NAME;
				break;
			default:
				templateName = FRAME_TEMPLATE_CLASS_NAME;
				break;
			}
		}
		try {
			generatedCode = generateCode(monitor, templateName);
		    saveCode(monitor, generatedCode, this.info.getClassName());
		} catch (JETException e) {
			e.printStackTrace();
		} catch (CoreException e) {
			e.printStackTrace();
		}
		
	}

	public void setModelSmellInfo(ModelSmellInfo info){
		this.info = info;
	}

	private IContainer findOrCreatePackage(IProgressMonitor progressMonitor) throws CoreException {
		IPath outputPath = new Path(this.info.getProjectName() + SOURCE_DIR + this.info.getPackage().replace('.', '/'));
		IProgressMonitor subMonitor = new SubProgressMonitor(progressMonitor, 1);
		IPath localLocation = null; 
		IContainer container = CodeGenUtil.EclipseUtil.findOrCreateContainer(outputPath, true, localLocation, subMonitor);
		return container;
	}

	private String generateCode(IProgressMonitor monitor, String template) {
		String templatePath = templateDirectory + template + TEMPLATE_FILE_EXTENSION;
		ClassLoader classLoader = getClass().getClassLoader();
		JETEmitter jetEmitter = new JETEmitter(templatePath, classLoader);
		jetEmitter.getClasspathEntries().addAll(classpathEntries);
		String result = "";
		IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 1);
		try {
			result = jetEmitter.generate(subMonitor, new Object[] {this.info});
		} catch (JETException e) {
			e.printStackTrace();
		}
		System.out.println(result);
		return result;
	}
	
	private List<IClasspathEntry> getVariableEntries() {
		ArrayList<IClasspathEntry> classpathEntries = new ArrayList<IClasspathEntry>();
//		Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID); 
//		String requires = (String) bundle.getHeaders().get(Constants.BUNDLE_CLASSPATH);
//		ManifestElement[] elements = null;
//		try {
//			elements = ManifestElement.parseHeader(Constants.BUNDLE_CLASSPATH, requires);
//		} catch (BundleException e) {
//			e.printStackTrace();
//		}
//		ManifestElement element = elements[0];
//		URL url = bundle.getEntry(element.getValue());
//		if (url != null) {
//			try {
//				IPath path = new Path(FileLocator.toFileURL(url).getFile());
//				if (!path.equals(JavaCore.getClasspathVariable(EMFMODELSMELLSJAR))) {
//					JavaCore.setClasspathVariable(EMFMODELSMELLSJAR, path,	null);
//					//TODO: EMFMODELSMELLSJAR richtig definieren
//				}
//			} catch (MalformedURLException e) {
//				e.printStackTrace();
//			} catch (JavaModelException e) {
//				e.printStackTrace();
//			} catch (IOException e) {
//				e.printStackTrace();
//			}
//			classpathEntries.add(JavaCore.newVariableEntry(new Path(EMFMODELSMELLSJAR), null, null));
//		}
		Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID);
		String version = (String) bundle.getHeaders().get(BUNDLEVERSION);
		classpathEntries.add(JavaCore.newLibraryEntry(new Path(PLUGINSPATH
				+ Activator.PLUGIN_ID + "_" + version + ".jar"), null, null));
		return classpathEntries;
	}

	private void saveCode(IProgressMonitor monitor, String content, String fileName) throws CoreException, JETException{
		IContainer container = findOrCreatePackage(monitor);
		if (container == null) {
			throw new JETException("Cound not find or create container for package " + this.info.getPackage() + " in " + this.info.getProjectName());
		}
		IFile file = container.getFile(new Path(fileName + JAVA_FILE_EXTENSION));
		System.out.println("saving code into: "+file.getLocation());
		InputStream inputStream = new ByteArrayInputStream(content.getBytes());
		if (!file.exists()) {
			file.create(inputStream, false, monitor);
		} else {
			container.refreshLocal(1, monitor);
			file.setContents(inputStream, true, false, monitor);
		}
		
	}
	
	private String setTemplateDirectory() {
		String directory = "";
		final Bundle bundle = Activator.getDefault().getBundle();
		try {
			directory = FileLocator.toFileURL(bundle.getEntry(TEMPLATE_DIR)).getFile();
		} catch (final IOException e) {
			e.printStackTrace();
		}
		return directory;
	}
	
	
	
}
