package de.unimarburg.swt.emf.modelsmell.wizards;

import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

import de.unimarburg.swt.emf.modelsmell.EMFModelSmells;
import de.unimarburg.swt.emf.modelsmell.model.generator.GenerationType;
import de.unimarburg.swt.emf.modelsmell.model.generator.HenshinModelSmellInfo;
import de.unimarburg.swt.emf.modelsmell.model.generator.MetricBasedModelSmellInfo;
import de.unimarburg.swt.emf.modelsmell.model.generator.ModelSmellInfo;
import de.unimarburg.swt.emf.modelsmell.wizards.pages.HenshinDataWizardPage;
import de.unimarburg.swt.emf.modelsmell.wizards.pages.MetricBasedDataWizardPage;
import de.unimarburg.swt.emf.modelsmell.wizards.pages.ModelSmellDataPage;

/**
 * The wizard for the user input regarding the new generation of smell plugins 
 * 
 * @author Matthias Burhenne
 *
 */

public class NewSmellWizard extends Wizard implements INewWizard{
	
	public static final String TRANSFORMATIONS_DIR = "/transformations/";
	private final String WINDOW_TITLE = "New Model Smell";
	private ModelSmellDataPage dataPage;
	private HenshinDataWizardPage henshinPage;
	private MetricBasedDataWizardPage metricPage;
	private LinkedList<IProject> projects;
	private IStructuredSelection initialSelection;
	private String newSmellId;
	private String newSmellName;
	private String newSmellMetamodel;
	private String newSmellDescription;
	private IProject newSmellTargetProject;
	private GenerationType generationType;
	
	
	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		initialSelection = selection;
		generationType = GenerationType.SKELETON;
		initProjects();		
	}

	@Override
	public void addPages() {
		setWindowTitle(WINDOW_TITLE);
		dataPage = new ModelSmellDataPage();
		henshinPage = new HenshinDataWizardPage();
		metricPage = new MetricBasedDataWizardPage();
		addPage(dataPage);
		addPage(henshinPage);
		addPage(metricPage);
	}

	@Override
	public boolean canFinish() {
		if(generationType == GenerationType.SKELETON)
			return dataPage.isPageComplete();
		if(generationType == GenerationType.METRIC_BASED)
			return dataPage.isPageComplete() && metricPage.isPageComplete();
		if(generationType == GenerationType.HENSHIN_TRANSFORMATION)
			return dataPage.isPageComplete() && henshinPage.isPageComplete();
		return false;
	}

	@Override
	public IWizardPage getStartingPage() {
		return dataPage;
	}

	@Override
	public boolean performFinish() {
		try{
			getContainer().run(true, true, new IRunnableWithProgress(){
				public void run(IProgressMonitor monitor)throws InvocationTargetException, InterruptedException{
					EMFModelSmells.createNewModelSmell(monitor, getModelSmellInfo(), newSmellTargetProject);
				}
			});
		}
		catch(InvocationTargetException e){
			e.printStackTrace();
			return false;
		}
		catch(InterruptedException e){
			e.printStackTrace();
			return false;
		}
		return true;
	}

	private void initProjects(){
		this.projects = new LinkedList<IProject>();
		IProject[] allProjects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (IProject project : allProjects) {
			if (project.isOpen()) {
				IProjectNature nature = null;
				try {
					nature = project.getNature("org.eclipse.pde.PluginNature");
				} catch (CoreException e) {
					e.printStackTrace();
				}
				if (null != nature) 
					this.projects.add(project);
			}
		}
	}
	
	private ModelSmellInfo getModelSmellInfo(){
		ModelSmellInfo info;
		switch (generationType) {
		case SKELETON:
			info = new ModelSmellInfo(newSmellId, newSmellName, newSmellDescription, 
					newSmellMetamodel, newSmellTargetProject, generationType);
			break;
		case HENSHIN_TRANSFORMATION:
			info = new HenshinModelSmellInfo(newSmellId, newSmellName, newSmellDescription, 
					newSmellMetamodel, newSmellTargetProject, generationType, henshinPage.getTransf());
			break;
		case METRIC_BASED:
			info = new MetricBasedModelSmellInfo(newSmellId, newSmellName, newSmellDescription, 
					newSmellMetamodel, newSmellTargetProject, generationType, 
					metricPage.getMetricData().getMetricID(), metricPage.getComparator());
			break;
		default:
			info = new ModelSmellInfo(newSmellId, newSmellName, newSmellDescription, 
					newSmellMetamodel, newSmellTargetProject, generationType);
			break;
		}
		return info;
	}


	public String getNewSmellId() {
		return newSmellId;
	}

	public void setNewSmellId(String newSmellId) {
		this.newSmellId = newSmellId;
	}

	public String getNewSmellName() {
		return newSmellName;
	}

	public void setNewSmellName(String newSmellName) {
		this.newSmellName = newSmellName;
	}

	public String getNewSmellMetamodel() {
		return newSmellMetamodel;
	}

	public void setNewSmellMetamodel(String newSmellMetamodel) {
		this.newSmellMetamodel = newSmellMetamodel;
	}

	public IProject getNewSmellTargetProject() {
		return newSmellTargetProject;
	}
	
	public LinkedList<IProject> getProjects(){
		return this.projects;
	}

	public void setNewSmellTargetProject(IProject newSmellTargetProject) {
		this.newSmellTargetProject = newSmellTargetProject;
	}

	public void setNewSmellTargetProject(String projectName) {
		for(IProject project : projects){
			String path = project.getName();
			if(path.equals(projectName))
				setNewSmellTargetProject(project);
		}
		
	}

	public void setNewSmellDescription(String text) {
		this.newSmellDescription = text;
		
	}
	
	public void setGenerationType(GenerationType type){
		this.generationType = type;
	}
	
	public GenerationType getGenerationType(){
		return this.generationType;
	}
	
	public HenshinDataWizardPage getHenshinPage(){
		return this.henshinPage;
	}
	
	public MetricBasedDataWizardPage getMetricPage(){
		return metricPage;
	}
	
	public ModelSmellDataPage getDataPage(){
		return dataPage;
	}

}
