/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.config;

import java.io.File;
import java.io.IOException;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl;
import org.eclipse.emf.refactor.common.core.EmfRefactoring;
import org.eclipse.emf.refactor.common.core.IController;
import org.eclipse.emf.refactor.common.core.ui.IGuiHandler;
import org.eclipse.emf.refactor.common.groupmanagement.EmfRefactoringEntry;
import org.eclipse.emf.refactor.common.groupmanagement.Group;
import org.eclipse.emf.refactor.common.groupmanagement.GroupManagement;
import org.eclipse.emf.refactor.common.groupmanagement.GroupmanagementFactory;
import org.eclipse.emf.refactor.common.groupmanagement.MetaModelGroup;
import org.eclipse.emf.refactor.common.groupmanagement.impl.GroupManagementImpl;
import org.eclipse.emf.refactor.common.groupmanagement.impl.GroupmanagementPackageImpl;
import org.eclipse.emf.refactor.common.projectgroupmapping.ProjectGroupMapping;
import org.eclipse.emf.refactor.common.projectgroupmapping.ProjectGroupMappingLoader;

/**
 * Configuration class of Emf Refactor.
 * @generated NOT
 * @author Florian Mantz
 */
public final class EmfRefactorConfig {
	
	/**
	 * Singleton instance. 
	 */
	private static EmfRefactorConfig instance;
		
	/**
	 * Set of all registered emf model refactorings. 
	 */
	private final SortedSet<EmfRefactoring> refactorings = 
									new TreeSet<EmfRefactoring>();
	
	/**
	 * GroupManagement object used for managing model refactoring
	 * groups.
	 */
	private GroupManagement groupManagement;
	
	/**
	 * Mapping of the actual project to several EMF model refactoring groups.
	 */
	private ProjectGroupMapping projectGroupMapping;
	
	private final String GROUPMANAGEMENTFILENAME = 
								"/configuration/.emfrefactorgroupmanagement";
	private final String DEFAULTGROUP = 
								"Default Group"; 
	private static final String REFACTORING_EXTENSION_POINT = 
								"org.eclipse.emf.refactor.common.Refactoring";
	
	/**
	 * Private constructor.
	 */
	private EmfRefactorConfig(){	
		this.refactorings.addAll(EmfRefactorConfig.loadRefactorings());
		if (this.groupManagement == null) {
			GroupmanagementPackageImpl.init();
			loadGroupManagement();
		}
	}
	
	/**
	 * Loads a list of EmfRefactoring objects out of the descriptions of
	 * the corresponding extension point servings.
	 * @return Created list of EmfRefactoring objects.
	 */
	private static SortedSet<EmfRefactoring> loadRefactorings() {
		SortedSet<EmfRefactoring> r = new TreeSet<EmfRefactoring>();
		IConfigurationElement[] rawRefactorings = 
				Platform.getExtensionRegistry()
					.getConfigurationElementsFor(REFACTORING_EXTENSION_POINT);		
		for(IConfigurationElement e:rawRefactorings){
			try {
				if(e.getName().equals("refactoring")){
					EmfRefactoring ref = createEmfRefactoring(e);
					if(null != ref)r.add(ref);
				}				
			} catch (Exception e2) {
				e2.printStackTrace();
			}
		}
		return r;
	}
	
	/**
	 * Creates an EmfRefactoring object out of the description of
	 * the corresponding extension point serving.
	 * @param rawRefactoring description of the corresponding extension 
	 * point serving. 
	 * @return Created EmfRefactoring object.
	 */
	private static EmfRefactoring createEmfRefactoring
									(IConfigurationElement rawRefactoring){	
		try {
			final String id = rawRefactoring.getAttribute("id");
			final String name = rawRefactoring.getAttribute("menulabel");
			IController controller = null;
			IGuiHandler gui = null;
			final String controllerClassName = 
								rawRefactoring.getAttribute("controller");
			final String guiClassName = 
								rawRefactoring.getAttribute("gui");
			final String namespaceUri = 
								rawRefactoring.getAttribute("namespaceUri");		
			if(controllerClassName.equals(guiClassName)){				
				controller = (IController)rawRefactoring
								.createExecutableExtension("controller");
				gui = (IGuiHandler)controller;				
				return 
					new EmfRefactoring
								(id, name, namespaceUri, gui, controller);
				
			} else {
				//Only create gui, controller is lazy loaded:
				gui = (IGuiHandler)rawRefactoring
										.createExecutableExtension("gui");	
				return 
					new EmfRefactoring
								(id, name, namespaceUri, gui, rawRefactoring);			
			}	
		} catch (Throwable e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Method for initializing the GroupManagement object used for 
	 * managing model refactoring groups.
	 */
	private void loadGroupManagement() {
		String path = new File("").getAbsolutePath() + GROUPMANAGEMENTFILENAME;
		try {
			this.groupManagement = loadGroupManagement(path);
			synchronizeRefactoringEntries();
			saveGroupManagement();
		} catch(Exception e) {
			createGroupManagement();
			saveGroupManagement();
		}
	}
	
	/**
	 * Creates a new GroupManagement object (with a default group).
	 */
	private void createGroupManagement() {
		GroupmanagementFactory groupmanagementFactory = 
												GroupmanagementFactory.eINSTANCE;
		this.groupManagement = groupmanagementFactory.createGroupManagement();
		this.groupManagement.getGroups().add(createDefaultGroup());
	}
	
	/**
	 * Creates a default refactoring group consisting of all registered
	 * EMF model refactorings.
	 */
	private Group createDefaultGroup(){
		GroupmanagementFactory groupmanagementFactory = 
			GroupmanagementFactory.eINSTANCE;
		Group group = groupmanagementFactory.createGroup();
		group.setName(DEFAULTGROUP);
		Set<String> uris = getUris();
		for (String uri : uris) {
			MetaModelGroup mmgroup = 
			groupmanagementFactory.createMetaModelGroup();
			mmgroup.setNsUri(uri);
			group.getMmGroups().add(mmgroup);
			for (EmfRefactoring refactoring : refactorings){
				if (refactoring.getNamespaceUri().equals(uri)){
					EmfRefactoringEntry entry = 
					groupmanagementFactory.createEmfRefactoringEntry();
					entry.setId(refactoring.getId());
					entry.setName(refactoring.getName());
					entry.setAlias(refactoring.getName());
					entry.setMmgroup(mmgroup);
					mmgroup.getEntries().add(entry);
				}
			}
		}
		return group;
	}
	
	/**
	 * Updates the GroupManagament by the given refactoring group.
	 * @param group Refactoring group to be added or updated. 
	 */
	public void updateGroup(Group group) {
		for (Group g : this.groupManagement.getGroups()) {
			if (g.getName().equals(group.getName())) {
				this.groupManagement.getGroups().remove(g);
				break;
			}
		}
		this.groupManagement.getGroups().add(group);
	}
	
	/**
	 * Gets a set of all different namespaceUris with existing EMF model
	 * refactorings.
	 * @return
	 */
	private Set<String> getUris() {
		Set<String> uris = new TreeSet<String>();
		for (EmfRefactoring refactoring : refactorings){
			uris.add(refactoring.getNamespaceUri());
		}
		return uris;
	}
	
	/**
	 * Saves the GroupManagementObject.
	 */
	public void saveGroupManagement() {
		URI uri = URI.createFileURI(new File("").getAbsolutePath() 
													+ GROUPMANAGEMENTFILENAME);
		Resource res = new XMIResourceImpl(uri);
		res.getContents().add(this.groupManagement);		
		try {
			res.save(null);
		} catch (IOException e){
			e.printStackTrace();
		}
	}
	
	/**
	 * Synchronizes the stored GroupManagement object with the
	 * existing EMF model refactorings.
	 */
	private void synchronizeRefactoringEntries() {
		GroupmanagementFactory groupmanagementFactory = 
											GroupmanagementFactory.eINSTANCE;
		GroupManagement gmTemp = groupmanagementFactory.createGroupManagement();
		for (Group group : this.groupManagement.getGroups()){
			if (group.getName().equals(DEFAULTGROUP)){
				gmTemp.getGroups().add(createDefaultGroup());
				continue;
			}
			Group gTemp = groupmanagementFactory.createGroup();
			gTemp.setName(group.getName());
			gmTemp.getGroups().add(gTemp);			
			for (MetaModelGroup mmGroup : group.getMmGroups()){
				MetaModelGroup mmgTemp = 
								groupmanagementFactory.createMetaModelGroup();
				mmgTemp.setNsUri(mmGroup.getNsUri());						
				for (EmfRefactoringEntry entry : mmGroup.getEntries()) {										
					if (isRefactoring(entry)){
						EmfRefactoringEntry entryTemp = 
							groupmanagementFactory.createEmfRefactoringEntry();
						entryTemp.setId(entry.getId());
						entryTemp.setName(entry.getName());
						entryTemp.setAlias(entry.getAlias());
						entryTemp.setMmgroup(mmgTemp);
						mmgTemp.getEntries().add(entryTemp);
					}
				}				
				if (! mmgTemp.getEntries().isEmpty()) {
					gTemp.getMmGroups().add(mmgTemp);
				}				
			}			
		}		
		this.groupManagement = gmTemp;
	}
	
	/**
	 * Checks whether the given entry corresponds to an existing
	 * emf model refactoring.
	 * @param entry Existing entry in GroupManagement.
	 * @return true if the refactoring exists, false otherwise.
	 */
	private boolean isRefactoring(EmfRefactoringEntry entry) {
		for (EmfRefactoring refactoring : this.refactorings){
			if ((entry.getMmgroup().getNsUri().equals
										(refactoring.getNamespaceUri())) 
					&& entry.getId().equals(refactoring.getId())
					&& entry.getName().equals(refactoring.getName())){
				return true;
			}
		}
		return true;
	}
	
	/**
	 * Loads the GroupManagement object from the specified filename.
	 * @param filename Name of the file of the GroupManagement object.
	 * @return GroupManagement object from the specified filename.
	 */
	private GroupManagementImpl loadGroupManagement(String filename) {
		ResourceSet resourceSet = new ResourceSetImpl();
		URI uri = URI.createFileURI(filename);
		Resource resource = resourceSet.getResource(uri, true);
		return (GroupManagementImpl)resource.getContents().get(0);
	}
	
	/**
	 * Returns the singleton instance.
	 * @return
	 */
	public static EmfRefactorConfig getInstance() {
		if(null == instance ){
			instance = new EmfRefactorConfig();
		}
		return instance;
	}
	
	/**
	 * Gets a set of all registered emf model refactorings. 
	 * @return Set of all registered emf model refactorings. 
	 */
	public SortedSet<EmfRefactoring> getRefactorings() {
		return refactorings;
	}
	
	/**
	 * Gets the GroupManagement object used for managing model refactoring
	 * groups.
	 * @return GroupManagement object used for managing model refactoring
	 * groups.
	 */
	public GroupManagement getGroupManagement() {
		return this.groupManagement;
	}
	
	/**
	 * Loads the ProjectGroupMapping for the given project. Either it can be 
	 * loaded from an existing file or it is newly created.
	 * @param iProject Project the ProjectGroupMapping has to be loaded for.
	 */
	public void loadProjectGroupMappings(IProject iProject) { 
		if (null == this.projectGroupMapping
				|| null == iProject 
				|| !(iProject.getName().equals
						(this.projectGroupMapping.getProjectName()))){
			if (null != iProject && 
						ProjectGroupMappingLoader.refConfigExists(iProject)){
				this.projectGroupMapping = 
						ProjectGroupMappingLoader.loadGroupConfig(iProject);
			} else {
				this.projectGroupMapping = new ProjectGroupMapping
							((null!=iProject)?iProject.getName():"no project");
				for (Group group : this.groupManagement.getGroups()){
					this.projectGroupMapping.addGroupName(group.getName());
				}
			}
		}
	}
	
	/**
	 * Synchronizes the existing ProjectGroupMapping with the existing
	 * model refactoring groups.
	 */
	public void synchronizeGroupMappings() {
		ProjectGroupMapping pgm = 
			new ProjectGroupMapping(this.projectGroupMapping.getProjectName());
		for (String groupName : this.projectGroupMapping.getGroupNames()) {
			for (Group group : this.groupManagement.getGroups()) {
				if (group.getName().equals(groupName)) {
					pgm.addGroupName(groupName);
					break;
				}
			}
		}
		this.projectGroupMapping = pgm;
	}
	
	/**
	 * Saves the actual ProjectGroupMapping in a file in the given project.
	 * @param iProject
	 */
	public void saveProjectGroupMapping(IProject iProject) {
		ProjectGroupMappingLoader
				.saveGroupConfig(this.projectGroupMapping, iProject);
	}
	
	/**
	 * Checks whether the refactoring group with the given name is mapped
	 * to the actual project.
	 * @param groupName Name of the group to be checked.
	 * @return true if the group with the given name is mapped, false otherwise.
	 */
	public boolean isValidGroup(String groupName) {
		boolean isValid = false;
		for (String gn : this.projectGroupMapping.getGroupNames()){
			if (gn.equals(groupName)){
				isValid = true;
				break;
			}
		}
		return isValid;
	}
	
	/**
	 * Sets the mapping of the actual project to several EMF model 
	 * refactoring groups.
	 * @param pgm Mapping of the actual project to several EMF model 
	 * refactoring groups.
	 */
	public void setProjectGroupMapping(ProjectGroupMapping pgm) {
		this.projectGroupMapping = pgm;
	}
	
	/**
	 * Gets the mapping of the actual project to several EMF model 
	 * refactoring groups.
	 * @return Mapping of the actual project to several EMF model 
	 * refactoring groups.
	 */
	public ProjectGroupMapping getProjectGroupMapping(){
		return this.projectGroupMapping;
	}
	
}
