/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.core;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.emf.refactor.common.config.EmfRefactorConfig;
import org.eclipse.emf.refactor.common.core.ui.IGuiHandler;

/**
 * Base class of a specific emf model refactoring.
 * @generated NOT
 * @author Florian Mantz
 */
public class EmfRefactoring implements Comparable<EmfRefactoring> {

	/**
	 * Id of the emf model refactoring.
	 */
	private final String id;
	
	/**
	 * Name of the emf model refactoring.
	 */
	private final String name;
	
	/**
	 * Namespace of the emf model refactoring.
	 */
	private final String namespaceUri;
	
	/**
	 * GuiHandler object of the emf model refactoring.
	 */
	private final IGuiHandler gui;
	
	/**
	 * Controller object of the emf model refactoring.
	 */
	private IController controller;
	
	/**
	 * ConfigurationElement object of the emf model refactoring used
	 * for lazy loading.
	 */
	private IConfigurationElement rawRefactoring;
	
	/**
	 * Contructs an emf model refactoring with GuiHandler object.
	 * @param id Id of the emf model refactoring.
	 * @param name Name of the emf model refactoring.
	 * @param namespaceUri Namespace of the emf model refactoring.
	 * @param gui GuiHandler object of the emf model refactoring.
	 */
	private EmfRefactoring
		(String id, String name, String namespaceUri,IGuiHandler gui) {
		this.id = id;
		this.name = name;
		this.namespaceUri = namespaceUri;
		this.gui = gui;
		this.gui.setParent(this);
	}
	
	/**
	 * Contructs an emf model refactoring with GuiHandler and a controller
	 * object.
	 * @param id Id of the emf model refactoring.
	 * @param name Name of the emf model refactoring.
	 * @param namespaceUri Namespace of the emf model refactoring.
	 * @param gui GuiHandler object of the emf model refactoring.
	 * @param controller Controller object of the emf model refactoring.
	 */
	public EmfRefactoring
		(String id, String name, String namespaceUri, 
				IGuiHandler gui, IController controller){
			this(id,name,namespaceUri,gui);
			this.controller = controller;
			this.controller.setParent(this);
	}
	
	/**
	 * Contructs an emf model refactoring with GuiHandler and a 
	 * ConfigurationElement object.
	 * @param id Id of the emf model refactoring.
	 * @param name Name of the emf model refactoring.
	 * @param namespaceUri Namespace of the emf model refactoring.
	 * @param gui GuiHandler object of the emf model refactoring.
	 * @param rawRefactoring ConfigurationElement object of the emf model 
	 * refactoring. 
	 */
	public EmfRefactoring
		(String id, String name, String namespaceUri, 
				IGuiHandler gui, IConfigurationElement rawRefactoring){
			this(id,name,namespaceUri,gui);
			this.rawRefactoring = rawRefactoring;
	}
	
	/**
	 * Gets the namespace of the emf model refactoring.
	 * @return Namespace of the emf model refactoring.
	 */
	public String getNamespaceUri(){
		return namespaceUri;
	}
	
	/**
	 * Gets the id of the emf model refactoring.
	 * @return Id of the emf model refactoring.
	 */
	public String getId() {
		return id;
	}

	/**
	 * Gets the name of the emf model refactoring.
	 * @return Name of the emf model refactoring.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Gets the controller object of the emf model refactoring.
	 * @return Controller object of the emf model refactoring.
	 */
	public IController getController() {
		lazyLoadController();
		return controller;
	}
	
	private void lazyLoadController() {
		if(null == controller){
			try {
				this.controller = 
					(IController)rawRefactoring
							.createExecutableExtension("controller");
				this.controller.setParent(this);
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Gets the guiHandler object of the emf model refactoring.
	 * @return GuiHandler object of the emf model refactoring.
	 */
	public IGuiHandler getGui() {
		return gui;
	}
	
	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EmfRefactoring other = (EmfRefactoring) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(EmfRefactoring emfRefactoring) {
		if(this.equals(emfRefactoring))return 0;
		if(this.getName().equals(emfRefactoring.getName())){
			return emfRefactoring.id.hashCode() - this.id.hashCode();
		}else{
		    return emfRefactoring.getName().compareTo(this.getName());	
		}
	}
	
	/**
	 * Returns the emf model refactoring with the specified id.
	 * @param id Id of the emf model refactoring to be returned.
	 * @return Emf model refactoring with the specified id.
	 */
	public static EmfRefactoring getById(String id){
		for (EmfRefactoring r : 
						EmfRefactorConfig.getInstance().getRefactorings()){
			if(r.getId().equals(id)){
				return r;
			}
		}
		throw 
			new IllegalArgumentException("Refactoring id does not exist!");
	}

	/**
	 * Return a list of all registered emf model refactorings.
	 * @return List of all registered emf model refactorings.
	 */
	public static List<EmfRefactoring> getRefactorings(){
		return new LinkedList<EmfRefactoring>
						(EmfRefactorConfig.getInstance().getRefactorings());
	}

}
