/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.groupmanagement;

import org.eclipse.emf.refactor.common.config.EmfRefactorConfig;
import org.eclipse.emf.refactor.common.groupmanagement.ui.GroupManagementPreferencePage;
import org.eclipse.emf.refactor.common.groupmanagement.ui.GroupManagementRefactoringDialog;
import org.eclipse.jface.dialogs.MessageDialog;

/**
 * Controller used for managing the group management process.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class GroupManagementController {
	
	/**
	 * GroupManagement object to be controlled.
	 */
	private GroupManagement groupManagement;
	
	/**
	 * Actual model refactoring group.
	 */
	private Group group;
	
	/**
	 * Actual model refactoring group name.
	 */
	private String groupName;
	
	/**
	 * Actual modus in the group management process.
	 */
	private GroupManagementModus modus;
	
	/**
	 * Main EMF model refactoring configurator.
	 */
	private EmfRefactorConfig config;
	
	/**
	 * PreferencePage of EMF Refactor.
	 */
	private GroupManagementPreferencePage groupManagementPreferencePage;
	
	/**
	 * Default EMF model refactoring group name for new groups.
	 */
	private final String DEFAULTNAME = "MyRefactoringGroup";
	
	/**
	 * Name of the default EMF model refactoring group.
	 */
	public final String DEFAULTGROUP = "Default Group"; 
	
	/**
	 * Default constructor invoked from the given PreferencePage.
	 * @param groupManagementPreferencePage PreferencePage from which
	 * the controller is invoked.
	 */
	public GroupManagementController(GroupManagementPreferencePage groupManagementPreferencePage) {
		this.config = EmfRefactorConfig.getInstance();
		this.groupManagement = this.config.getGroupManagement();
		this.groupManagementPreferencePage = groupManagementPreferencePage;
	}
	
	/**
	 * Gets the PreferencePage from which the controller is invoked.
	 * @return PreferencePage from which the controller is invoked.
	 */
	public GroupManagementPreferencePage getGroupManagementPreferencePage() {
		return this.groupManagementPreferencePage;
	}
	
	/**
	 * Gets the actual model refactoring group name.
	 * @return Actual model refactoring group name.
	 */
	public String getGroupName(){
		return this.groupName;
	}
	
	/**
	 * Sets the actual model refactoring group name.
	 * @param groupName Actual model refactoring group name.
	 */
	public void setGroupName(String groupName){
		this.groupName = groupName;
	}
	
	/**
	 * Gets the GroupManagement object to be controlled.
	 * @return GroupManagement object to be controlled.
	 */
	public GroupManagement getGroupManagement(){
		return this.groupManagement;
	}
	
	/**
	 * Sets the actual modus in the group management process.
	 * @param modus Actual modus in the group management process.
	 */
	private void setModus(GroupManagementModus modus){
		this.modus = modus;
	}
	
	/**
	 * Method for starting the refactoring group management process 
	 * depenedent on the actual modus.  Actual modus in the group management 
	 * process.
	 * @param groupManagementModus 
	 */
	public void startModus(GroupManagementModus groupManagementModus){
		setModus(groupManagementModus);
		switch (this.modus) {
			case NEWGROUP: 
				groupManagementPreferencePage.startNaming(DEFAULTNAME);
				break;				
			case SHOWGROUP: 
				groupManagementPreferencePage.startSelection();
				break;
			case DELETEGROUP:
				groupManagementPreferencePage.startSelection();
				break;
			case UPDATEGROUP: 
				groupManagementPreferencePage.startSelection();
				break;
			case RENAMEGROUP:
				groupManagementPreferencePage.startSelection();
				break;     
			default:
				MessageDialog.openError(null, "EMF Refactor", 
								"Error in managing refactoring groups!");
		}
	}
	
	/**
	 * Method for checking whether a model refactoring group with the
	 * given name exists.
	 * @param name Model refactoring group name to be checked.
	 * @return true if a a model refactoring group with the given name 
	 * exists, false otherwise.
	 */
	public boolean existsGroup(String name) {
		for (Group g : groupManagement.getGroups()){
			if (g.getName().equals(name)){
				if (modus.equals(GroupManagementModus.RENAMEGROUP)){
					return (!group.getName().equals(name));
				} else {
					return true;
				}
			}
		}
		return false;		
	}
	
	/**
	 * Method for continuing the group management process after
	 * editing a group name.
	 */
	public void continueBeyondNameDialog() {
		switch (this.modus) {
			case NEWGROUP: 
				new GroupManagementRefactoringDialog(this);
				break;				
			case RENAMEGROUP:
				group.setName(groupName);
				config.saveGroupManagement();
				MessageDialog.openInformation(null, "EMF Refactor", 
								"New Group '" + groupName + "' saved.");
				groupManagementPreferencePage.reset();
				break;     
			case SHOWGROUP: 
			case DELETEGROUP:
			case UPDATEGROUP: 
			default:
				MessageDialog.openError(null, "EMF Refactor", 
								"Error in managing refactoring groups!");
		}
	}
	
	/**
	 * Returns the default model refactoring group named "DefaultGroup".
	 * @return Default model refactoring group. 
	 */
	public Group getDefaultGroup() {
		Group defaultGroup = GroupmanagementFactory.eINSTANCE.createGroup();
		for (Group group : groupManagement.getGroups()){
			if (group.getName().equals(DEFAULTGROUP)){
				defaultGroup = group;
				break;
			}
		}
		return defaultGroup;
	}
	
	/**
	 * Method for continuing the group management process after
	 * editing the model refactorings.
	 * @param group Actual EMF model refactoring group.
	 */
	public void continueBeyondRefactoringDialog(Group group) {
		switch (this.modus) {
			case NEWGROUP: 
				groupManagement.getGroups().add(group);
				config.saveGroupManagement();
				groupManagementPreferencePage.reset();
				break;				
			case UPDATEGROUP:
				config.updateGroup(group);
				config.saveGroupManagement();
				groupManagementPreferencePage.reset();
				break;
			case SHOWGROUP: 
				groupManagementPreferencePage.reset();
				break;
			case RENAMEGROUP:
			case DELETEGROUP:
			default:
				MessageDialog.openError(null, "EMF Refactor", 
								"Error in managing refactoring groups!");
		}
	}
	
	/**
	 * Method for continuing the group management process after
	 * editing the model refactoring group name.
	 * @param groupName Actual name of the model refactoring group.
	 */
	public void continueBeyondSelection(String groupName) {
		groupManagementPreferencePage.stopSelection();
		setGroupName(groupName);
		setGroupByName();
		switch (this.modus) {
			case UPDATEGROUP:
				new GroupManagementRefactoringDialog(this, group, true);
				break;
			case RENAMEGROUP:
				groupManagementPreferencePage.startNaming(groupName);
				break;
			case SHOWGROUP: 
				new GroupManagementRefactoringDialog(this, group, false);
				break;
			case DELETEGROUP:
				if (MessageDialog.openConfirm(null, 
						"EMF Refactor - Delete Refactoring Group", 
						"Do you really want to delete group '" + 
											this.group.getName() + "'?")) {
					this.groupManagement.getGroups().remove(this.group);
					config.saveGroupManagement();
					MessageDialog.openInformation(null, 
							"EMF Refactor", "Group '" + group.getName() 
															+ "' deleted.");
				}
				groupManagementPreferencePage.reset();
				break;
			case NEWGROUP: 
			default:
				MessageDialog.openError(null, "EMF Refactor", 
									"Error in managing refactoring groups!");
		}
	}
	
	/**
	 * Sets the actual group according to the actual group name.
	 */
	private void setGroupByName() {
		for (Group g : groupManagement.getGroups()) {
			if (g.getName().equals(this.groupName)){
				this.group = g;
			}
		}
	}
	
	/** 
	 * Gets the actual modus of the group management process.
	 * @return Actual modus of the group management process.
	 */
	public GroupManagementModus getModus() {
		return this.modus;
	}	

}
