/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.projectgroupmapping;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.IProject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Static class for loading and storing a ProjectGroupManagement.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class ProjectGroupMappingLoader {
	
	private static final String REF_CONFIG = ".emfrefactor.xml";
	private static final String PROJECT_GROUP_MAPPING_TAG = 
											"ProjectGroupMapping";
	private static final String PROJECT_NAME_TAG = "ProjectName";
	private static final String GROUPNAME_ENTRY_TAG = "GroupName";
	
	/**
	 * Checks whether a ProjectGroupMapping file exists in the
	 * given project.
	 * @param iProject Project to be checked whether a ProjectGroupMapping
	 * file exists.
	 * @return true if a ProjectGroupMapping file exists in the
	 * given project, false otherwise.
	 */
	public static boolean refConfigExists(IProject iProject) {
		java.net.URI uri = iProject.getLocationURI();
		String path = uri.getPath() + "/" + 
								ProjectGroupMappingLoader.REF_CONFIG;
		File file = new File(path);
		return file.exists();
	}
	
	/**
	 * Loads the ProjectGroupMapping file from the given project.
	 * @param iProject Project from that the ProjectGroupMapping file has
	 * to be load.
	 * @return ProjectGroupMapping object of the loaded file.
	 */
	public static ProjectGroupMapping loadGroupConfig(IProject iProject) {
		final DocumentBuilder builder = 
						ProjectGroupMappingLoader.newDocumentBuilder();
		ProjectGroupMapping projectGroupMapping = null;
		if (null != builder){
			Document doc = null;
			try {
				java.net.URI uri = iProject.getLocationURI();
				doc = builder.parse(uri + "/" + 
									ProjectGroupMappingLoader.REF_CONFIG);
			} catch (final SAXException e) {
				e.printStackTrace();
			} catch (final IOException e) {
				e.printStackTrace();
			}
			if (doc != null) {
				Element root = doc.getDocumentElement();
				NodeList projectList = root.getElementsByTagName
								(ProjectGroupMappingLoader.PROJECT_NAME_TAG);
				Element project = (Element) projectList.item(0);
				projectGroupMapping = new ProjectGroupMapping
												(project.getTextContent());
				NodeList entryList = root.getElementsByTagName
							(ProjectGroupMappingLoader.GROUPNAME_ENTRY_TAG);
				for (int i=0; i < entryList.getLength(); i++){
					Element entry = (Element) entryList.item(i);
					projectGroupMapping.addGroupName(entry.getTextContent());
				}
			}		
		}
		return projectGroupMapping;
	}
	
	/**
	 * Saves the given ProjectGroupMapping object in a ProjectGroupMapping file
	 * inside the given project.
	 * @param projectGroupMapping ProjectGroupMapping to be saved.
	 * @param iProject Project the ProjectGroupMapping has to be saved in.
	 */
	public static void saveGroupConfig(ProjectGroupMapping projectGroupMapping, 
															IProject iProject) {
		final DocumentBuilder builder = 
							ProjectGroupMappingLoader.newDocumentBuilder();
		if (null != builder){
			final Document doc = builder.newDocument();			
			final Element root = doc.createElement
						(ProjectGroupMappingLoader.PROJECT_GROUP_MAPPING_TAG);
			doc.appendChild(root);			
			final Element projectName = doc.createElement
						(ProjectGroupMappingLoader.PROJECT_NAME_TAG);
			projectName.setTextContent(projectGroupMapping.getProjectName());
			root.appendChild(projectName);			
			for (String groupName : projectGroupMapping.getGroupNames()){
				Element groupNameEntry = doc.createElement
						(ProjectGroupMappingLoader.GROUPNAME_ENTRY_TAG);
				groupNameEntry.setTextContent(groupName);
				root.appendChild(groupNameEntry);
			}			
			final TransformerFactory transformerFactory = 
												TransformerFactory.newInstance();
			Transformer transformer = null;
			try {
				transformer = transformerFactory.newTransformer();
			} catch (final TransformerConfigurationException e) {
				e.printStackTrace();
			}
			final DOMSource source = new DOMSource(doc);			
			java.net.URI uri = iProject.getLocationURI();
			StreamResult result = new StreamResult
							(uri + "/" + ProjectGroupMappingLoader.REF_CONFIG);			
			try {
				transformer.transform(source, result);
			} catch (final TransformerException e) {
				e.printStackTrace();
			}			
		}		
	}
	
	private static DocumentBuilder newDocumentBuilder() {
		try {
			return DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (final ParserConfigurationException e) {
			e.printStackTrace();
			return null;
		}
	}

}
