/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.generator.util;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.refactor.generator.RefactoringConfig;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Abstract class for load and store operations (plugin.xml file and
 * RefactoringConfig out of model refactoring extension point.
 * @generated NOT
 * @author Thorsten Arendt
 */
public abstract class RefactoringGeneratorUtil {
	
	private static final String PLUGINXML = "plugin.xml";
	private static final String PLUGIN = "plugin";
	private static final String EXTENSION = "extension";
	private static final String POINT = "point";
	private static final String EXT_POINT_REF_ID = 
							"org.eclipse.emf.refactor.common.Refactoring";
	private static final String REFACTORING = "refactoring";
	private static final String CONTROLLER = "controller";
	private static final String GUIHANDLER = "gui";
	private static final String REFACTORINGID = "id";
	private static final String MENULABEL = "menulabel";
	private static final String NAMESPACEURI = "namespaceUri";	

	/**
	 * Static method for checking whether the project with the given name
	 * owns a plugin.xml file.
	 * @param projectName Name of the project to be checked.
	 * @return true if the project with the given name owns a plugin.xml 
	 * file; false otherwise.
	 */
	public static boolean pluginXmlExists(String projectName) {
		IProject project = ResourcesPlugin.getWorkspace()
											.getRoot().getProject(projectName);
		java.net.URI uri = project.getLocationURI();
		String path = uri.getPath() + "/" + RefactoringGeneratorUtil.PLUGINXML;
		File file = new File(path);
		return file.exists();
	}

	/**
	 * Static method for creating a plugin.xml file within the project
	 * with the given name.
	 * @param projectName Name of the project the plugin.xml file has
	 * to be generated into.
	 */
	public static void createPluginXml(String projectName) {
		final DocumentBuilder builder = 
							RefactoringGeneratorUtil.newDocumentBuilder();
		if (null != builder){
			final Document doc = builder.newDocument();			
			final Element root = doc.createElement
										(RefactoringGeneratorUtil.PLUGIN);
			doc.appendChild(root);
			final TransformerFactory transformerFactory = 
										TransformerFactory.newInstance();
			Transformer transformer = null;
			try {
				transformer = transformerFactory.newTransformer();
				transformer.setOutputProperty(OutputKeys.INDENT,"yes");
			} catch (final TransformerConfigurationException e) {
				e.printStackTrace();
			}
			final DOMSource source = new DOMSource(doc);
			IProject project = ResourcesPlugin.getWorkspace().getRoot()
												.getProject(projectName);
			java.net.URI uri = project.getLocationURI();
			StreamResult result = new StreamResult
						(uri + "/" + RefactoringGeneratorUtil.PLUGINXML);			
			try {
				transformer.transform(source, result);
			} catch (final TransformerException e) {
				e.printStackTrace();
			}			
		}
	}	
	
	private static DocumentBuilder newDocumentBuilder() {
		try {
			return DocumentBuilderFactory.newInstance()
												.newDocumentBuilder();
		} catch (final ParserConfigurationException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Abstract method for getting a list of RefactoringConfig out from
	 * the model refactoring extension point served in the plugin.xml 
	 * file of the project with the given name.
	 * @param projectName Name of the project the plugin.xml file has to
	 * be analyzed for serving the model refactoring extension point.
	 * @return List of RefactoringConfig out from the model refactoring
	 * extension point served in the plugin.xml file of the project with
	 * the given name.
	 */
	public static List<RefactoringConfig> getRefactoringConfig
												(String projectName) {
		final DocumentBuilder builder = RefactoringGeneratorUtil
												.newDocumentBuilder();
		List<RefactoringConfig> refactoringConfigs = 
									new ArrayList<RefactoringConfig>();
		if (null != builder){
			Document doc = null;
			try {
				IProject project = ResourcesPlugin.getWorkspace()
									.getRoot().getProject(projectName);
				java.net.URI uri = project.getLocationURI();
				doc = builder.parse
						(uri + "/" + RefactoringGeneratorUtil.PLUGINXML);
			} catch (final SAXException e) {
				e.printStackTrace();
			} catch (final IOException e) {
				e.printStackTrace();
			}
			if (doc != null) {
				Element root = doc.getDocumentElement();
				NodeList refactorings = root.getElementsByTagName
									(RefactoringGeneratorUtil.REFACTORING);
				for (int i=0; i < refactorings.getLength(); i++){
					Element refactoring = (Element) refactorings.item(i);
					String controller = refactoring.getAttribute
									(RefactoringGeneratorUtil.CONTROLLER);
					String gui = refactoring.getAttribute
									(RefactoringGeneratorUtil.GUIHANDLER);
					String id = refactoring.getAttribute
									(RefactoringGeneratorUtil.REFACTORINGID);
					String menuLabel = refactoring.getAttribute
									(RefactoringGeneratorUtil.MENULABEL);
					String namespaceURI = refactoring.getAttribute
									(RefactoringGeneratorUtil.NAMESPACEURI);
					RefactoringConfig refactoringConfig = 
							new RefactoringConfig
								(controller, gui, id, menuLabel, namespaceURI);
					refactoringConfigs.add(refactoringConfig);
				}				
			}
		}
		return refactoringConfigs;
	}

	/**
	 * Abstract method for saving the given RefactoringConfig list as model
	 * refactoring extension point servings in the plugin.xml file of the
	 * project with the given name. 
	 * @param projectName Name of the project the model refactoring 
	 * extension point servings shall be stored in the plugin.xml file.
	 * @param refactoringConfig List of RefactoringConfig to be saved as
	 * model refactoring extension point servings.
	 */
	public static void saveRefactoringConfig
			(String projectName, List<RefactoringConfig> refactoringConfig) {
		final DocumentBuilder builder = 
							RefactoringGeneratorUtil.newDocumentBuilder();
		if (null != builder){
			final Document doc = builder.newDocument();			
			Element root = doc.createElement(RefactoringGeneratorUtil.PLUGIN);
			doc.appendChild(root);
			final Element extension = doc.createElement
										(RefactoringGeneratorUtil.EXTENSION);
			extension.setAttribute(RefactoringGeneratorUtil.POINT, 
									RefactoringGeneratorUtil.EXT_POINT_REF_ID);
			for (RefactoringConfig rc : refactoringConfig){
				Element refactoring = doc.createElement
										(RefactoringGeneratorUtil.REFACTORING);
				if (rc.getPackageName() == null){
					refactoring.setAttribute(RefactoringGeneratorUtil.CONTROLLER, 
								rc.getController());
					refactoring.setAttribute(RefactoringGeneratorUtil.GUIHANDLER, 
								rc.getGuiHandler());
				} else {
					refactoring.setAttribute(RefactoringGeneratorUtil.CONTROLLER, 
								rc.getPackageName() + "." + rc.getController());
					refactoring.setAttribute(RefactoringGeneratorUtil.GUIHANDLER, 
								rc.getPackageName() + "." + rc.getGuiHandler());
				}
				refactoring.setAttribute(RefactoringGeneratorUtil.REFACTORINGID, 
								rc.getRefactoringId());
				refactoring.setAttribute(RefactoringGeneratorUtil.MENULABEL, 
								rc.getMenuLabel());
				refactoring.setAttribute(RefactoringGeneratorUtil.NAMESPACEURI, 
								rc.getNamespaceUri());
				extension.appendChild(refactoring);
			}
			root.appendChild(extension);
			final TransformerFactory transformerFactory = 
												TransformerFactory.newInstance();
			Transformer transformer = null;
			try {
				transformer = transformerFactory.newTransformer();
				transformer.setOutputProperty(OutputKeys.INDENT,"yes");
			} catch (final TransformerConfigurationException e) {
				e.printStackTrace();
			}
			final DOMSource source = new DOMSource(doc);
			IProject project = ResourcesPlugin.getWorkspace()
											.getRoot().getProject(projectName);
			java.net.URI uri = project.getLocationURI();
			StreamResult result = new StreamResult(uri + "/" 
											+ RefactoringGeneratorUtil.PLUGINXML);			
			try {
				transformer.transform(source, result);
			} catch (final TransformerException e) {
				e.printStackTrace();
			}			
		}
	}

}
