/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.henshin.generator.ui;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.refactor.henshin.generator.RefactoringConfigHenshin;
import org.eclipse.emf.refactor.henshin.generator.RefactoringGeneratorHenshin;
import org.eclipse.jface.wizard.IWizardContainer;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

/**
 * Wizard for editing data needed for model refactoring code generation
 * using Henshin.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class CreationWizardHenshin extends Wizard {
	
	/**
	 * Name of the project the model refactoring code has to be 
	 * generated to.
	 */
	private String projectName; 
	
	/**
	 * Name of the Henshin file used for executing the EMF model
	 * refactoring.
	 */
	private String transformationFileName;
	
	/**
	 *  Wizard page for configurating model refactoring data (basics).
	 */
	private BasicsWizardPage basicWizardPage;
	
	/**
	 * Wizard page for specifying Henshin files used for initial checking,
	 * final checking, and executing the EMF model refactoring.
	 */
	private TransformationWizardPage transformationWizardPage;
	
	/**
	 * Wizard page for configurating model refactoring parameters.
	 */
	private ParameterWizardPage parameterWizardPage;
	
	/**
	 * Wizard page for configurating model test parameters.
	 */
	private TestWizardPage testWizardPage;
	
	/**
	 * Default constructor that initializes its wizard pages.
	 * @param selectedEObject The selected EObject when starting the 
	 * generation process of the model refactoring.
	 */
	public CreationWizardHenshin(EObject selectedEObject){
		super();
		setWindowTitle("EMF Refactor - Specify EMF Model Refactoring " +
				"										using Henshin");
		this.projectName = "";
		this.basicWizardPage = 
			new BasicsWizardPage("EMF Refactor", selectedEObject, this);
		this.transformationWizardPage = 
			new TransformationWizardPage("EMF Refactor", this);
		this.parameterWizardPage = 
			new ParameterWizardPage("EMF Refactor");
		this.testWizardPage =
			   new TestWizardPage("EMF Refactor");
		this.addPage(basicWizardPage);
		this.addPage(transformationWizardPage);
		this.addPage(parameterWizardPage);
		this.addPage(testWizardPage);
	}
	
	/**
	 * Sets the name of the project the model refactoring code has to be 
	 * generated to.
	 * @param projectName Name of the project the model refactoring code 
	 * has to be generated to.
	 */
	public void setProjectName(String projectName){
		this.projectName = projectName;
		this.transformationWizardPage.setProjectName(projectName); 
		this.parameterWizardPage.setProjectName(projectName); 
	}
	
	/**
	 * Sets the name of the Henshin file used for executing the EMF model
	 * refactoring.
	 * @param transformationFileName Name of the Henshin file used for 
	 * executing the EMF model refactoring.
	 */
	public void setTransformationFileName(String transformationFileName) {
		this.transformationFileName = transformationFileName;
		this.parameterWizardPage.setTransformationFileName(transformationFileName); 
	}

	/** Gets the name of the Henshin file used for executing the EMF model
	 * refactoring.
	 * @return Name of the Henshin file used for executing the EMF model
	 * refactoring.
	 */
	public String getTransformationFileName() {
		return transformationFileName;
	}
	
	/**
	 * Gets the name of the project the model refactoring code has to be 
	 * generated to.
	 * @return Name of the project the model refactoring code has to be 
	 * generated to.
	 */
	public String getProjectName(){
		return this.projectName;
	}

	/**
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			protected void execute(IProgressMonitor monitor)
					throws CoreException, InvocationTargetException,
					InterruptedException {
				try {
					createRefactoring(monitor);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			IWizardContainer container = getContainer();
			if (container != null) {
				container.run(false, false, op);
			}
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return true;
	}
	
	/**
	 * Triggers the model refactoring generation activity. Collects data
	 * from its wizard pages and starts the refactoring generator.
	 * @param monitor Object monitoring the model refactoring 
	 * generation activity.
	 */
	protected void createRefactoring(IProgressMonitor monitor) {
		String projectName = basicWizardPage.getProjectName();
		String id = basicWizardPage.getRefactoringId();
		String menuLabel = basicWizardPage.getMenuLabel();
		String namespaceUri = basicWizardPage.getNsUri();
		String createChangeFileName = 
				transformationWizardPage.getTransformationFileName();
		String namespacePrefix = basicWizardPage.getNsPrefix();
		int numberOfTests = testWizardPage.getNumberOfTests();
		RefactoringConfigHenshin refactoringConfig = 
			new RefactoringConfigHenshin(projectName, id, menuLabel, 
									namespaceUri, createChangeFileName, 
									namespacePrefix, numberOfTests);
		refactoringConfig.setCheckInitialFileName
					(transformationWizardPage.getCheckInitialFileName());
		refactoringConfig.setCheckFinalFileName
					(transformationWizardPage.getCheckFinalFileName());
		refactoringConfig.setSelectedEObjectClass
					(basicWizardPage.getClassName());
		refactoringConfig.setSelectedEObjectJar
					(basicWizardPage.getJar());
		refactoringConfig.setParameters
					(parameterWizardPage.getSelectedParameters());
		refactoringConfig.setMetaModelName
					(basicWizardPage.getMetaModelName());
		RefactoringGeneratorHenshin rg = 
					new RefactoringGeneratorHenshin(refactoringConfig);
		rg.run(monitor);
	}

}
