/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.henshin.runtime;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.henshin.interpreter.EmfEngine;
import org.eclipse.emf.henshin.interpreter.RuleApplication;
import org.eclipse.emf.henshin.interpreter.UnitApplication;
import org.eclipse.emf.henshin.interpreter.interfaces.InterpreterEngine;
import org.eclipse.emf.henshin.interpreter.util.ModelHelper;
import org.eclipse.emf.henshin.matching.EmfGraph;
import org.eclipse.emf.henshin.model.HenshinFactory;
import org.eclipse.emf.henshin.model.Parameter;
import org.eclipse.emf.henshin.model.TransformationSystem;
import org.eclipse.emf.henshin.model.TransformationUnit;
import org.eclipse.emf.refactor.henshin.core.IHenshinDataManagement;
import org.eclipse.emf.refactor.henshin.core.IHenshinInformation;

/**
 * Class that executes Henshin transformations using the Henshin
 * interpreter.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class HenshinRunner {
	
	/**
	 * Given selection of EObjects the model refactoring 
	 * shall be executed on.
	 */
	protected final List<EObject> selection;
	
	public final static String HENSHINEXTENSION = ".henshin";
	private static final String MAINUNIT = "mainUnit";
	
	/**
	 * Corresponding HenshinDataManagement object which specifies 
	 * data concerning a model refactoring that shall be executed by 
	 * EMF model transformations using Henshin. 
	 */
	private IHenshinDataManagement dataManagementObject;
	
	/**
	 * Factory for creating Henshin objects.
	 */
	private static HenshinFactory henshinFactory = HenshinFactory.eINSTANCE;
	
	/**
	 * Henshin transformation system to be executed.
	 */
	private TransformationSystem transformationSystem = 
								henshinFactory.createTransformationSystem();

	/**
	 * Default constructor using a selection of EObjects the model refactoring 
	 * shall be executed on and the corresponding HenshinDataManagement object 
	 * which specifies data concerning a model refactoring that shall be 
	 * executed by EMF model transformations using Henshin. 
	 * @param selection Given selection of EObjects the model refactoring 
	 * shall be executed on.
	 * @param dataManagementObject Corresponding HenshinDataManagement object 
	 * which specifies data concerning a model refactoring that shall be 
	 * executed by EMF model transformations using Henshin. 
	 */
	public HenshinRunner
		(List<EObject> selection, IHenshinDataManagement dataManagementObject){
		this.selection = selection;
		this.dataManagementObject = dataManagementObject;
	}
	
	/**
	 * Static method for checking initial or final conditions of a EMF
	 * model refactoring using Henshin transformations.
	 * @param henshinInformation Corresponding HenshinInformation object 
	 * specifying a Henshin file that shall be used.
	 * @param root Root object of the EMF model to be refactored.
	 * @return List of messages indicating errors which occur during
	 * conditions checking.
	 */
	public static List<String> checkConditions
					(IHenshinInformation henshinInformation, EObject root) {
		List<String> result = new ArrayList<String>();
		if (henshinInformation.isTransformationFileSpecified()) {
			String path = henshinInformation.getTransformationFileName() 
														+ HENSHINEXTENSION;
			TransformationSystem checkTransformationSystem = 
						(TransformationSystem) ModelHelper.loadFile(path);
			EmfGraph graph = new EmfGraph();
			graph.addRoot(root);
			InterpreterEngine engine = new EmfEngine(graph);
			TransformationUnit mainUnit = 
						checkTransformationSystem.findUnitByName(MAINUNIT);
			Map <Parameter, Object> values = new HashMap<Parameter, Object>();
			for (Parameter par : mainUnit.getParameters()){
				try {
					values.put(par, 
							henshinInformation.getHenshinDataManagement()
								.getInPortByName(par.getName()).getValue());
				} catch (IllegalArgumentException ex) {
					ex.printStackTrace();
				}			
			}
			UnitApplication unitApplication = 
							new UnitApplication (engine, mainUnit, values);
			unitApplication.execute();
			Stack<RuleApplication> appliedRules = 
										unitApplication.getAppliedRules();
			while (!appliedRules.empty()) {
				RuleApplication appliedRule = appliedRules.pop();
				if ((null != appliedRule.getDescription()) 
							&& (! appliedRule.getDescription().isEmpty())) {
					result.add
						(appliedRule.getMatch().getRule().getDescription());
				}
			}
		}
		return result;
	}
	
	/**
	 * Method that executes the EMF model refactoring using Henshin.
	 */
	public void run(){
		if (this.dataManagementObject.getCreateChangeHenshinInformation()
										.isTransformationFileSpecified()) {
			String path = this.dataManagementObject
					.getCreateChangeHenshinInformation()
						.getTransformationFileName() + HENSHINEXTENSION;
			this.transformationSystem = 
						(TransformationSystem) ModelHelper.loadFile(path);
			EObject root = EcoreUtil.getRootContainer(selection.get(0));
			EmfGraph graph = new EmfGraph();
			graph.addRoot(root);
			InterpreterEngine engine = new EmfEngine(graph);
			TransformationUnit mainUnit = 
						this.transformationSystem.findUnitByName(MAINUNIT);
			Map <Parameter, Object> values = new HashMap<Parameter, Object>();
			for (Parameter par : mainUnit.getParameters()){
				try {
					values.put
						(par, this.dataManagementObject
								.getInPortByName(par.getName()).getValue());
				} catch (IllegalArgumentException ex) {
					ex.printStackTrace();
				}			
			}
			UnitApplication unitApplication = 
							new UnitApplication (engine, mainUnit, values);
			unitApplication.execute();
		}
	}
}
