/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.runtime.config;

import java.util.Comparator;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import org.eclipse.core.resources.IProject;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.refactor.common.config.EmfRefactorConfig;
import org.eclipse.emf.refactor.common.core.EmfRefactoring;
import org.eclipse.emf.refactor.common.groupmanagement.EmfRefactoringEntry;
import org.eclipse.emf.refactor.common.groupmanagement.Group;
import org.eclipse.emf.refactor.common.groupmanagement.MetaModelGroup;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

/**
 * Configuration class of the runtime module of EMF Refactor.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class RuntimeConfig {
	
	/**
	 * Flag for supporting EMF model refactoring on GMF generated 
	 * graphical editors. Default = true.
	 * TODO Configurable in Settings:
	 */
	private boolean supportGmfDiagramEditors = true;
	
	/**
	 * Flag for indicating whether the namespaceUri of the model
	 * has to be checked against the namespaceUri of the EMF model
	 * refactoring. Default = true.
	 * TODO Configurable in Settings:
	 */
	private boolean checkEmfVersion = true;
	
	/**
	 * Instance of the main configuration class of 
	 * EMF Refactor.
	 */
	private EmfRefactorConfig config = EmfRefactorConfig.getInstance();
	
	/**
	 * Returns whether the namespaceUri of the model has to be checked 
	 * against the namespaceUri of the EMF model refactoring.
	 * @return true if the namespaceUri of the model has to be checked 
	 * against the namespaceUri of the EMF model refactoring, false
	 * otherwise.
	 */
	public boolean isCheckEmfVersion() {
		return checkEmfVersion;
	}

	/**
	 * Sets whether the namespaceUri of the model has to be checked 
	 * against the namespaceUri of the EMF model refactoring.
	 * @param checkEmfVersion Flag for indicating whether the namespaceUri 
	 * of the model has to be checked against the namespaceUri of the EMF 
	 * model refactoring.
	 */
	public void setCheckEmfVersion(boolean checkEmfVersion) {
		this.checkEmfVersion = checkEmfVersion;
	}
	
	/**
	 * Returns whether GMF generated graphical editors are supported
	 * by EMF model refactorings.
	 * @return true if GMF generated graphical editors are supported,
	 * false otherwise.
	 */
	public boolean isSupportGmfDiagramEditors() {
		return supportGmfDiagramEditors;
	}

	/**
	 * Sets whether GMF generated graphical editors shall be supported
	 * by EMF model refactorings.
	 * @param supportUML2Tools Flag for supporting EMF model refactorings 
	 * on GMF generated graphical editors.
	 */
	public void setSupportGmfDiagramEditors(boolean supportUML2Tools) {
		this.supportGmfDiagramEditors = supportUML2Tools;
	}
	
	/**
	 * Method that returns the alias name of a EMF model 
	 * refactoring with the given id.
	 * @param id Id of the EMF model refactoring.
	 * @return Alias name of a EMF model refactoring with the given id.
	 */
	public String getAliasOfId(String id) {
		String alias = "";
		for (Group group : config.getGroupManagement().getGroups()){
			if (config.isValidGroup(group.getName())) {
				for (MetaModelGroup mmGroup : group.getMmGroups()) {
					for (EmfRefactoringEntry entry : mmGroup.getEntries()) {	
						if (entry.getId().equals(id)){
							if (null != entry.getAlias() 
											&& ! entry.getAlias().isEmpty()) {
								alias = entry.getAlias();
							} else {
								alias = entry.getName();
							}
							break;
						}
					}
					if (! alias.isEmpty()) break;
				}
			}
			if (! alias.isEmpty()) break;
		}
		return alias;
	}
	
	/**
	 * Gets a list of EMF model refactorings applicable on the given
	 * selection elements. 
	 * @param selection Given selection objects the model refactoring 
	 * shall be executed on.
	 * @return List of EMF model refactorings.
	 */
	public SortedSet<EmfRefactoring> getProjectRefactorings
												(List<EObject> selection){
		SortedSet<EmfRefactoring> ret = new TreeSet<EmfRefactoring>(
				new Comparator<EmfRefactoring>(){
					@Override
					public int compare(EmfRefactoring o1, EmfRefactoring o2) {
						int i = getAliasOfId(o2.getId())
										.compareTo(getAliasOfId(o1.getId()));
						return (0 == i)?o2.getId().compareTo(o1.getId()):i;
					}
				});
		String nsElement = getNsUriFromSelection(selection);
		for(EmfRefactoring r : config.getRefactorings()){
			if(this.isValidRefactoring(r) && 
					(!this.checkEmfVersion 
							|| r.getNamespaceUri().equals(nsElement))){
				ret.add(r);
			}
		}
		return ret;
	}
	
	/**
	 * Returns whether the given EMF model refactoring can be applied 
	 * according to the mapped model refactoring group(s). 
	 * @param refactoring EMF model refactoring to be checked.
	 * @return true if the refactoring can be applied, false otherwise.
	 */
	private boolean isValidRefactoring(EmfRefactoring refactoring){
		boolean isValid = false;
		List<String> groupNames = 
						config.getProjectGroupMapping().getGroupNames();
		for (String groupName : groupNames){
			Group group = getGroupByName(groupName);
			for (MetaModelGroup mmg : group.getMmGroups()) {
				for (EmfRefactoringEntry refactoringEntry : 
												mmg.getEntries()) {
					if (refactoringEntry.getId()
									.equals(refactoring.getId())){
						isValid = true;
						break;
					}
				}
				if (isValid) break;
			}
			if (isValid) break;
		}
		return isValid;
	}
	
	/**
	 * Gets the model refactoring group with the given name.
	 * @param groupName Name of the model refactoring group.
	 * @return Model refactoring group with the given name.
	 */
	private Group getGroupByName(String groupName) {
		Group group = null;
		for (Group g : config.getGroupManagement().getGroups()) {
			if (g.getName().equals(groupName)) {
				group = g;
				break;
			}
		}
		return group;
	}
	
	/**
	 * Gets the namespaceUri of the given selection objects.
	 * @param selection Given selection objects the model refactoring 
	 * shall be executed on.
	 * @return NamespaceUri of the given selection objects. If there
	 * is no namespaceUri an empty String will be returned.
	 */
	private String getNsUriFromSelection(List<EObject> selection){
		String nsElement = "";
		if(checkEmfVersion && 0 < selection.size()){
			try {
				nsElement = ((EPackage)selection.get(0).eClass().eContainer())
																	.getNsURI();				
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return nsElement;
	}
	
	/**
	 * Static method that returns the project in the workspace
	 * according to the actual editor in the workbench. 
	 * @return Project in the workspace according to the actual 
	 * editor in the workbench. 
	 */
	@SuppressWarnings("finally")
	public static IProject getActualProject(){
		IProject actualProject = null;
		IWorkbenchWindow window = 
						PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		try {
			IEditorPart  editorPart = window.getActivePage().getActiveEditor();
			if ( editorPart != null ) {
				IEditorInput input = editorPart.getEditorInput();
				if (input instanceof IFileEditorInput) {
					IFileEditorInput fileInput = (IFileEditorInput) input;
					actualProject = fileInput.getFile().getProject();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			return actualProject;
		}
	}
	
	/**
	 * Checks whether the given class has the given name or whether one
	 * of its interfaces has the given name. This method is expensive! 
	 * Do not use it to often!
	 * @param cl Class to be checked.
	 * @param iname Name to be checked.
	 * @param isInterface Flag that indicates that the given name is a
	 * name of an interface.
	 * @return true if the given class has the given name or if one
	 * of its interfaces has the given name, false otherwise.
	 */
	public static boolean checkIsTypeOf
					(Class<?> cl, String iname, boolean isInterface){
		Class<?> superclass = null;
		if(cl.getName().equals(iname)){
			return true;
		}else if(isInterface){
			for(Class<?> c:cl.getInterfaces()){
				if(checkIsTypeOf(c, iname, isInterface)){
					return true;
				}
			}
		}
		if(null != (superclass = cl.getSuperclass())){
			return checkIsTypeOf(superclass, iname, isInterface);
		}else{
			return false;
		}
	}

}
